#ifndef _DYNAMICARRAY_H_
#define _DYNAMICARRAY_H_

#include <string.h>
#include <string>

using namespace std;

//=============================================================================
// Spezialklasse fr Aushnahmebehandlungen (Exception Handling)
//=============================================================================
class DynamicArrayOutOfRange
{
  public:
    //-------------------------------------------------------------------------
    // Constructor
    //-------------------------------------------------------------------------
    DynamicArrayOutOfRange (const char* pText, int pIndex)
    {
      sText  = pText;
      nIndex = pIndex;
    }

    //-------------------------------------------------------------------------
    // Fehlertext als C Zeiochenkette abrufen
    //-------------------------------------------------------------------------
    const char *getCStr (void)
    {
      return sText.c_str();
    }

    //-------------------------------------------------------------------------
    // Fehlertext als C++ string abrufen
    //-------------------------------------------------------------------------
    string getString (void)
    {
      return sText;
    }

    //-------------------------------------------------------------------------
    // Abfrage des fehlgeschlagenen Indexes
    //-------------------------------------------------------------------------
    int getIndex (void)
    {
      return nIndex;
    }

  private:
    //-------------------------------------------------------------------------
    // Standard-Constructor (DEAKTIVIERT)
    //-------------------------------------------------------------------------
    DynamicArrayOutOfRange () {}

    //-------------------------------------------------------------------------
    // Membervariablen
    //-------------------------------------------------------------------------
    string sText;
    int    nIndex;
};

//=============================================================================
// Template-Klasse zur Erstellung eines dynamischen Arrays
//=============================================================================
template<class T> class DynamicArray
{
  public:
    //=========================================================================
    // Standard Constructor
    //=========================================================================
    DynamicArray<T>()
    {
      nAct  = 0;
      aList = NULL;
    }

    //=========================================================================
    // Constructor mit Grenvorgabe
    //=========================================================================
    DynamicArray<T>(int pSize)
    {
      nAct  = 0;
      aList = NULL;
      resize (pSize);
    }

    //=========================================================================
    // Destructor
    //=========================================================================
    ~DynamicArray<T>()
    {
      if (aList)
      {
        delete [] aList;
      }
    }

    //=========================================================================
    // operator[]
    //=========================================================================
    T& operator[] (int pIndex) const
    {
      if (aList == NULL || pIndex < 0 || pIndex >= nAct)
      {
        throw DynamicArrayOutOfRange("DynamicArray", pIndex);
      }
      return aList[pIndex];
    }

    //=========================================================================
    // operator=
    //=========================================================================
    DynamicArray<T>& operator= (DynamicArray<T>& pToCopy)
    {
      //-----------------------------------------------------------------------
      // Nur kopieren, wenn nicht dasselbe Objekt
      //-----------------------------------------------------------------------
      if (this != &pToCopy)
      {
        //---------------------------------------------------------------------
        // Wenn im Ziel etwas enthalten ist, dann lschen
        //---------------------------------------------------------------------
        if (aList) reset();

        if (pToCopy.aList)
        {
          nAct = pToCopy.nAct;
          aList = new T[pToCopy.nAct];
          for (int i=0; i<pToCopy.nAct; i++)
          {
            aList[i] = pToCopy.aList[i];
          }
        }
      }
      return *this;
    }

    //=========================================================================
    // reset
    //=========================================================================
    void reset (void)
    {
      if (aList)
      {
        delete [] aList;
        aList = NULL;
        nAct = 0;
      }
    }

    //=========================================================================
    // resize
    //=========================================================================
    void resize (int pSize)
    {
      if (pSize != nAct)
      {
        T *aListOld = aList;
        aList = new T[pSize];

        if (aListOld)
        {
          int nCount = pSize < nAct ? pSize : nAct;
          for (int i=0; i<nCount; i++) aList[i] = aListOld[i];
          delete [] aListOld;
        }
        nAct = pSize;
      }
    }

    //=========================================================================
    // add
    //=========================================================================
    void add (T& pAdd)
    {
      resize (nAct + 1);
      aList[nAct-1] = pAdd;
    }

    //=========================================================================
    // size
    //=========================================================================
    int size (void) const
    {
      return nAct;
    }

  private:
    int  nAct;
    T   *aList;
};

#endif
